/*
 * Copyright 2021 Arne Vajhøj.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.jnosql.rocksdb;

import java.io.IOException;
import java.util.Properties;

import org.eclipse.jnosql.communication.Settings;
import org.eclipse.jnosql.communication.keyvalue.BucketManagerFactory;
import org.eclipse.jnosql.communication.keyvalue.KeyValueConfiguration;

import dk.vajhoej.jnosql.Deserialize;
import dk.vajhoej.jnosql.Serialize;

/**
 * Configuration for RocksDB.
 *
 */
public class RocksDBConfiguration implements KeyValueConfiguration {
	private String dbpath;
	private boolean prefix;
	private Serialize to;
	private Deserialize from;
	/**
	 * Construct configuration based on parameters.
	 * @param dbpath Database path
	 * @param prefix prefix keys with bucket name (true is compatible with JNoSQL code expecting buckets to be distinct, false is compatible with no-JNoSQL access to the database)
	 * @param to Serializer (Object -&gt; byte[])
	 * @param from Deserializer (byte[] -&gt; Object)
	 */
	public RocksDBConfiguration(String dbpath, boolean prefix, Serialize to, Deserialize from) {
		this.dbpath = dbpath;
		this.prefix = prefix;
		this.to = to;
		this.from = from;
	}
	/**
	 * Construct configuration based on rocksdb.properties with properties:
	 * <dl>
	 * <dt>dbpath</dt>
	 * <dd>Database path</dd>
	 * <dt>prefix</dt>
	 * <dd>prefix keys with bucket name (true/false)</dd>
	 * <dt>to</dt>
	 * <dd>Serializer class name</dd>
	 * <dt>from</dt>
	 * <dd>Deserializer class name</dd>
	 * </dl>
	 * @throws IOException if rocksdb.properties not found
	 * @throws ClassNotFoundException if serializer or deserializer class not found
	 * @throws IllegalAccessException if serializer or deserializer not accessible
	 * @throws InstantiationException if serializer or deserializer not instantiable
	 */
	public RocksDBConfiguration() throws InstantiationException, IllegalAccessException, ClassNotFoundException, IOException {
		Properties props = new Properties();
		props.load(RocksDBConfiguration.class.getResourceAsStream("/rocksdb.properties"));
		dbpath = props.getProperty("dbpath");
		prefix = Boolean.parseBoolean(props.getProperty("prefix"));
		to = (Serialize)Class.forName(props.getProperty("to")).newInstance();
		from = (Deserialize)Class.forName(props.getProperty("from")).newInstance();
	}
	/**
	 * Get BucketManager.
	 * @param settings Settings to use (key names are the same as in properties file, key values are the same as in constructor with arguments)
	 */
	@Override
	public BucketManagerFactory apply(Settings settings) {
		return new RocksDBBucketManagerFactory((String)settings.getOrDefault("dbpath", dbpath),
                                               (Boolean)settings.getOrDefault("prefix", prefix),
                                               (Serialize)settings.getOrDefault("to", to),
		                                       (Deserialize)settings.getOrDefault("from", from));
	}
}
